#!/usr/bin/perl -wT
#####################################################################
#
# WakeMeUp - an alternative (VDR) - wakeup solution with WoL
# (wakeup script [fallback option])
#
# (c)2008 Matthias Eckert <matzetronic at gmx dot de>
#
# Version 0.1  (2008-02-12)
#
#####################################################################


use strict;
use warnings;

use IO::Socket;
use Getopt::Long;


my $PROGNAME = "WakeUp";
my $PROGVERS = "0.1";
my $PROGDATE = "2008-02-12";



my $ipAddress;
my $port;
my $hwAddress;
my $magic;



sub usage {
    print STDOUT << "EOL";

Usage: wakeup [options] hardware_address

Available options:
  -h                         Print this help message.
  -i ip_address              Set the destination IP address.
  -p port                    Set the destination port (default is 9).


EOL
;
    exit 0;
}




##############
###  MAIN  ###
##############

print STDOUT "$PROGNAME $PROGVERS ($PROGDATE) by Matthias Eckert\n\n";

&Getopt::Long::Configure('pass_through', 'prefix_pattern=-');
my $result = GetOptions ( 'h'   => \&usage,
                          'i=s' => \$ipAddress,
                          'p=i' => \$port,
                         );

$hwAddress = shift(@ARGV);

if ($#ARGV > -1) {
    foreach (@ARGV) {
        print STDOUT "Unknown command line option '$_'.\n";
    }
    print STDOUT "See 'wakeup -h' for a list of available options.\n";
    exit 1;
}

if (! defined ($ipAddress)) {
    print STDERR "Option '-i' not given!\n";
    exit 1;
}
else {
    if (! $ipAddress || $ipAddress !~ /^(([01]?[0-9][0-9]?|2[0-4][0-9]|25[0-5])\.){3}([01]?[0-9][0-9]?|2[0-4][0-9]|25[0-5])$/) {
        print STDERR "Invalid value for '-i'!\n";
        exit 1;
    }
}

if (! defined ($hwAddress)) {
    print STDERR "HW address not given!\n";
    exit 1;
}
else {
    if (! $hwAddress || $hwAddress !~ /^([0-9A-F]{2,2}:){5,5}[0-9A-F]{2,2}$/i) {
        print STDERR "Invalid HW address!\n";
        exit 1;
    }
}

if (! defined ($port) || ! $port) {
    $port = 9;
}
else {
    if ($port !~ /^[\d]{1,5}$/ || $port > 65535) {
        print STDERR "Invalid value for '-p'!\n";
        exit 1;
    }
}

$ipAddress =~ /^([\d]+\.[\d]+\.[\d]+\.[\d]+)$/;
$ipAddress = $1;
$hwAddress =~ /^(.*)$/; # evil untaint
$hwAddress = $1;
$port =~ /^([\d]+)$/;
$port = $1;



my $handle = IO::Socket::INET->new( Proto         => 'udp',
                                    PeerHost      => $ipAddress,
                                    PeerPort      => $port,
                                   ) or die "$!";

foreach (split /:/, $hwAddress) {
    $magic .= chr(hex($_));
}
$magic = chr(0xFF) x 6 . $magic x 16;
$handle->send($magic);
$handle->close();
exit 0;



#####################################################################
#
# History:
#
# Version 0.1   (2008-02-12) me
# - initial version
#
#####################################################################
